/* listsort: prima versione */
#include <stdio.h>
#include <stdlib.h>
#include "intlist.h"

/* legge n numeri dal file fp e li carica nell'array a */
void readarray(FILE *fp, int *a, int n)
{
	int i;

	for(i = 0; i < n; i++)
		/* aritmetica dei puntatori: a + i equivale a &a[i] */
		fscanf(fp,"%d",a + i);
}

/* costruisce una lista contenente gli elementi dell'array a, */
/* tramite inserimento in testa */
intlist *buildlist(int *a, int n)
{
	int i;
	intlist *root = createlist();

	for(i = 0; i < n; i++)
		root = insert(root,a[i]);
	return root;
}

/* preleva l'elemento di testa della lista puntata da root e lo */
/* inserisce in ordine nella nuova lista ordinata sorted. */
/* Nota: dopo aver letto il valore numerico root->dato, elimina l'elemento stesso */
intlist *listsort(intlist *root)
{
	intlist *sorted;
	int k;

	sorted = createlist();
	while(root) {
		k = root->dato;
		root = delete(root,root);
		sorted = insertinorder(sorted,k);
	}
	return sorted;
 }

void stampa(intlist *p)
{
	printf("%d%c",p->dato, p->next ? ',' : '.');
}

int main(int ac, char **av)
{
	FILE *fp; 
	int n;
	int *array;
	intlist *root;
	
	if(ac != 3) {
		fprintf(stderr, "Uso: %s <file> <n>\n", av[0]);
		exit(-1);
	}
	if(!(fp = fopen(av[1],"r"))) {
		fprintf(stderr, "Errore: non riesco ad aprire il file %s\n", av[1]);
		exit(-2);
	}

	/* converte la stringa che rappresenta n nel numero intero n */
	n = atoi(av[2]);
	if(!n)
		return 0;

	if(!(array = calloc(n,sizeof(int)))) {
		fprintf(stderr, "Errore nell'allocazione della memoria\n");
		exit(-3);
	};
	readarray(fp,array,n);

	root = buildlist(array,n);
	free(array);
	printf("\nLista prima dell'ordinamento:\n");
	traverse(root,stampa);

	root = listsort(root);
	printf("\nLista ordinata:\n");
	traverse(root,stampa);

	return 0;
}
