/* listsort: seconda versione */
/* ordina la lista originaria spostando solo i collegamenti, */
/* senza allocare/deallocare alcun nodo */
#include <stdio.h>
#include <stdlib.h>
#include "intlist.h"

void readarray(FILE *fp, int *a, int n)
{
	int i;

	for(i = 0; i < n; i++)
		/* aritmetica dei puntatori: a + i equivale a &a[i] */
		fscanf(fp,"%d",a + i);
}

intlist *buildlist(int *a, int n)
{
	int i;
	intlist *root = createlist();

	for(i = 0; i < n; i++)
		root = insert(root,a[i]);
	return root;
}

/* inserisce un nodo gia' creato q in testa alla lista di radice p */
intlist *insertq(intlist *p, intlist *q)
{
	q->next = p;
	return q;
}

/* inserisce un nodo gia' creato q nella lista ordinata di radice p */
intlist *insertinorderq(intlist *p, intlist *q)
{	
	intlist *r;

	if(!p || p->dato >= q->dato)
		return insertq(p, q);
	for(r = p; r->next && r->next->dato < q->dato; r = r->next);
	r->next = insertq(r->next, q);
	return p;
}

/* ordina la lista di radice root restituendo la nuova radice della lista ordinata */
intlist *listsort(intlist *root)
{
	intlist *p,*q;

	if(!root)
		return NULL;

	/* isola il primo elemento dal resto della lista, */
	/* e scandisce la lista a partire dal secondo */
 	p = root->next;	
	root->next = NULL;

	/* cicla sugli elementi della lista, spostando la radice al prossimo elemento */
	/* e inserendo l'elemento corrente nella lista ordinata puntata da root */
	while(p) {
		q = p->next;
		root = insertinorderq(root,p);
		p = q;
	}
	return root;
}

void stampa(intlist *p)
{
	printf("%d%c",p->dato, p->next ? ',' : '.');
}

int main(int ac, char **av)
{
	FILE *fp;
	int n;
	int *array;
	intlist *root;
	
	if(ac != 3) {
		fprintf(stderr, "Uso: %s <file> <n>\n", av[0]);
		exit(-1);
	}
	if(!(fp = fopen(av[1],"r"))) {
		fprintf(stderr, "Errore: non riesco ad aprire il file %s\n", av[1]);
		exit(-2);
	}
	n = atoi(av[2]);
	if(!n)
		return 0;

	
	if(!(array = calloc(n,sizeof(int)))) {
		fprintf(stderr, "Errore nell'allocazione della memoria\n");
		exit(-3);
	};
	readarray(fp,array,n);

	root = buildlist(array,n);
	free(array);
	printf("\nLista prima dell'ordinamento:\n");
	traverse(root,stampa);

	root = listsort(root);
	printf("\nLista ordinata:\n");
	traverse(root,stampa);

	return 0;
}
